; +--------------+
; | DateBack.asm |
; +--------------+

; +---------------------------------------------------------------------------------------------------+
;	DateBack is a FASM assembly program which is designed to aid in backing up data to removable media
;	1) Process CommandLine to check for at least 2 passed parameters, a) backup drive b) at least one file to back up
;	2) Check for presence of backup media
;	3) Calculate approximate total size of files to be backed up
;	4) Compare this calculated size with available size on backup media
;		If enough room, proceed
;		If not enoush room, delete the oldest existing folder previously created by DateBack,
;			if able to increase available space, go to 4)
;	5) Create a new folder consisting of the YYYYMMDDdow, if a folder with that name already exists, the user is warned
;	6) Copy all files included in command line to the above folder
; +---------------------------------------------------------------------------------------------------+

format PE GUI 4.0

include '\Program Files\fasm\include\win32ax.inc'
include '\Program Files\fasm\include\macro\masm.inc'
include '\Program Files\fasm\include\GlobalMem.inc'
include	'DateBack.inc'

entry start

section '.code'		code readable executable
include	'utilities.asm'

align 4
start:
		invoke	GetModuleHandle, NULL
			mov	[hInstance], eax
			mov	[wc.hInstance], eax
;1)
		mov		[num_args], 0
		call	get_cl
		cmp		[num_args], 0
		jne		.clear_q
.disp_usage:
		invoke	MessageBox, NULL, MsgBoxText4, MsgBoxCaption, MB_OK
		invoke	ExitProcess, 0

.clear_q:
		call	clear_quotes

		call	OnlyOneInstance			;returns TRUE if we continue
			cmp		eax, TRUE
			je		.ooi				;continue
			invoke	ExitProcess, 0
.ooi:	mov		[str_buf], byte 0
		mov		esi, [cl_args]
		mov		edi, drive
		movsb							;Store first argument 'Drive'

		; +------------------------------+
		; | registering the window class |
		; +------------------------------+
		mov		[wc.lpfnWndProc], window_procedure
		mov		[wc.lpszClassName], wClsName
		mov		[wc.lpszMenuName], 30
		mov		[wc.hbrBackground], COLOR_WINDOW + 1
		invoke	LoadIcon, NULL, IDI_APPLICATION
			mov	[wc.hIcon], eax
		invoke	LoadCursor, NULL, IDC_ARROW
			mov	[wc.hCursor], eax
		invoke	RegisterClass, wc
		invoke	GetSystemMetrics, SM_CXFULLSCREEN    ;Fullscreen width
			mov	[fsWidth], eax
		invoke	GetSystemMetrics, SM_CYFULLSCREEN    ;Fullscreen heigth
			mov	[fsHeight], eax

		; +--------------------------+
		; | creating the main window |
		; +--------------------------+
		invoke	CreateWindowEx,\
				WS_EX_CLIENTEDGE,\
				wClsName,\
				wTitle,\
				WS_OVERLAPPEDWINDOW or WS_VISIBLE,\
				0,\
				0,\
				[fsWidth],\
				[fsHeight],\
				NULL,\
				NULL,\
				[hInstance],\
				NULL
			mov	[hWindow], eax
		invoke	GetMenu, eax			;30 - menuMain in resource section
			mov	[hMenu], eax
		invoke	ShowWindow, [hWindow], SW_SHOWNORMAL
		invoke	UpdateWindow, [hWindow]


		; +---------------------------+
		; | entering the message loop |
		; +---------------------------+
	window_message_loop_start:
		invoke	GetMessage, msg, NULL, 0, 0
			or	eax, eax
			je	window_message_loop_end
		invoke	TranslateMessage, msg
		invoke	DispatchMessage, msg
			jmp	window_message_loop_start

	window_message_loop_end:
		invoke	ExitProcess, 0

		; +----------------------+
		; | the window procedure |
		; +----------------------+
window_procedure	proc	hWnd:DWORD, uMsg:DWORD, wParam:DWORD, lParam:DWORD
locals
	cScroll	dd ?
	cArrow	dd ?
	cTitle	dd ?
	cBorder	dd ?
	cWidth	dd ?
	cHeight	dd ?
	lpBuffer dd ?
endl
	push	ebx esi edi
	cmp		[uMsg], WM_COMMAND
		je	wmCOMMAND
	cmp		[uMsg], WMU_THREAD_FINISH
		je	wmuTHREAD_FINISH
	cmp		[uMsg], WM_DESTROY
		je	wmDESTROY
	cmp		[uMsg], WM_CREATE
		je	wmCREATE
	cmp		[uMsg], WM_TIMER
		je	wmTIMER

wmDEFAULT:
	invoke	DefWindowProc, [hWnd], [uMsg], [wParam], [lParam]
		jmp	wmBYE

wmuTHREAD_FINISH:
	invoke	MessageBox, NULL, wTitle, wTitle, MB_OK
		jmp	wmBYE

wmCOMMAND:
	cmp		[wParam], 0xFFFF and MI_THREAD_CREATE
		je	wmCOMMAND_MI_THREAD_CREATE
	cmp		[wParam], 0xFFFF and MI_EXIT
		je	wmCOMMAND_MI_EXIT
	jmp		wmBYE

wmCOMMAND_MI_THREAD_CREATE:
	mov		eax, thread_procedure
	invoke	CreateThread, NULL, 0, eax, NULL, NORMAL_PRIORITY_CLASS, tId
	invoke	CloseHandle, eax
		jmp	wmBYE

wmCREATE:
	invoke	GetSystemMetrics, SM_CXVSCROLL
		mov	[cScroll], eax
	invoke	GetSystemMetrics, SM_CYVSCROLL
		mov	[cArrow], eax
	invoke	GetSystemMetrics, SM_CYSIZE
		mov	[cTitle], eax
	invoke	GetSystemMetrics, SM_CYSIZEFRAME
		shl	eax, 1
		mov	[cBorder], eax
	mov		eax, [fsWidth]
	sub		eax, [cScroll]
	add		eax, [cBorder]
	mov		[cWidth], eax
	mov		eax, [fsHeight]
	sub		eax, [cArrow]
	sub		eax, [cTitle]
	add		eax, [cBorder]
	mov		[cHeight], eax
	invoke	CreateWindowEx, \
				WS_EX_CLIENTEDGE, \
				EditClassName, \
				NULL, \
				WS_CHILD or WS_VISIBLE or WS_VSCROLL or WS_MAXIMIZE or \
				ES_LEFT or ES_AUTOVSCROLL or ES_MULTILINE \
				or ES_READONLY, \
				0, \
				0, \
				[cWidth], \
				[cHeight], \
				[hWnd], \
				EditID, \
				[hInstance], \
				NULL
	cmp		eax, 0
	jne		.goodh
		invoke	GetLastError
		lea		ebx, [lpBuffer]
		invoke	FormatMessage, \
					FORMAT_MESSAGE_ALLOCATE_BUFFER + FORMAT_MESSAGE_FROM_SYSTEM, \
					0, \
					eax, \
					LANG_NEUTRAL, \
					ebx, \
					0, \
					0
		invoke	MessageBox, \
					0, \
					[lpBuffer], \
					NULL, \
					MB_ICONERROR + MB_OK
		invoke	LocalFree, [lpBuffer]
.goodh:
	mov		[hwndEdit], eax
	invoke	SetFocus, [hwndEdit]
	invoke	SetTimer, [hWnd], 4321, 200, NULL
	jmp		wmBYE

wmTIMER:
	invoke	KillTimer, [hWnd], 4321
	call	SetUp
	cmp		eax, 0
	jne		wmDESTROY
;			mov		eax, thread_procedure
;			invoke	CreateThread, NULL, 0, eax, NULL, NORMAL_PRIORITY_CLASS, tId
;			invoke	CloseHandle, eax
	jmp		wmBYE

wmCOMMAND_MI_EXIT:
	invoke	DestroyWindow, [hWnd]
	jmp		wmBYE

wmDESTROY:
	invoke	PostQuitMessage, 0

wmBYE:
	pop		edi esi ebx
	ret
window_procedure	endp

SetUp	proc
locals
	BigSize		dd	?
	.lpBuffer	dd	?
	.spc		dd	?
	.bps		dd	?
	.free_clus	dd	?
	.tot_clus	dd	?
	.disk_left	dd	?
	.hFile		dd	?
	.OldErrorMode	dd	?
	.bud		rb	4				;BackUp Drive letter
	
	.sa			SECURITY_ATTRIBUTES
endl
	push	ebx edi esi
	mov		[highSize], 0
	mov		[lowSize], 0
	invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, ReqAct
	invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, crlf
	invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, DestDrive
	mov		edi, cl_args
	invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, dword [edi]
	invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, crlf
	invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, Files2BU
	push	[num_args]
	dec		[num_args]
.anot_file:
	add		edi, 4
	invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, dword [edi]
	invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, crlf
	invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, TwoTabs
	mov		[.sa.nLength], sizeof.SECURITY_ATTRIBUTES
	mov		[.sa.lpSecurityDescriptor], 0
	mov		[.sa.bInheritHandle], 0
	lea		eax, [.sa]
	invoke	CreateFile, dword [edi], GENERIC_READ, NULL, eax, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL
	mov		[.hFile], eax
	lea		ebx, [BigSize]
	invoke	GetFileSize, eax, ebx		;if file is > 
	add		[lowSize], eax
	mov		eax, [BigSize]
	adc		[highSize], eax
	invoke	CloseHandle, [.hFile]
	dec		[num_args]
	jne		.anot_file
	pop		[num_args]
.check_drive:
	invoke	SetErrorMode, SEM_FAILCRITICALERRORS
	mov		[.OldErrorMode], eax
	mov		edi, [cl_args]
	mov		al,  byte [edi]
	mov		[.bud], al
	mov		al,  byte [edi+1]
	mov		[.bud+1], al
	mov		[.bud+2], '\'
	mov		[.bud+3], 0
int 3
	invoke	GetDiskFreeSpaceEx, addr .bud, avail_bytes, total_bytes, free_bytes
	.if eax = 0
		invoke	MessageBox, [hWindow], MsgBoxText2, MsgBoxCaption, MB_OK
		invoke	SetErrorMode, [.OldErrorMode]
		mov		eax, 1
		jmp		.dir_out
	.endif
	invoke	SetErrorMode, [.OldErrorMode]
;	mov		edi, cl_args
	invoke	SetCurrentDirectory, addr .bud
	cmp		eax, TRUE
	je		.good_drive
		invoke	MessageBox, NULL, MsgBoxText2, MsgBoxCaption, MB_OK
		mov		eax, 1
		jmp		.dir_out
.good_drive:
	invoke	GetDiskFreeSpaceEx, addr .bud, avail_bytes, total_bytes, free_bytes
	invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, crlf
	invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, SpaceReq
	stdcall	jdwtoa, dword [lowSize], dword str_buf, 1
	invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, str_buf
	invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, crlf
	invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, SpaceAvail
	cmp		[avail_bytes.dwHighSize], 0
	jne		.gt_1
		stdcall	jdwtoa, dword [avail_bytes.dwLowSize], dword str_buf, 1
		invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, str_buf
		jmp		.lt_1
.gt_1:
	invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, GreaterThan
	jmp		.enough_room
.lt_1:
	invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, crlf
	invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, crlf
	mov		eax, [avail_bytes.dwLowSize]
	cmp		eax, [lowSize]
	ja		.enough_room
		call	FreeSome
		cmp		eax, 1
		jne		.enough_room
			invoke	MessageBox, [hwndEdit], NotEnough, N_E, MB_OK
			mov		eax, 1
			jmp		.dir_out
.enough_room:
	invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, crlf
	invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, FilesBBU
	invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, crlf
	call	get_dt		;creates date-based directory, eax != 0 IF SUCESSFUL
	cmp		eax, 0
	jne		.good_create
		invoke	MessageBox, [hwndEdit], szDirExists, wClsName, \
					MB_APPLMODAL or MB_OKCANCEL
		cmp		eax, IDCANCEL			;2 - Don't do backup
		jne		.good_create
			jmp		.dir_out
.good_create:
	call	ProcessFiles
.dir_out:
	pop		esi edi ebx
	ret
SetUp	endp

FreeSome	proc
locals
	.First_time	dd	?
	.hSearch	dd	?
	.sfoabort	dd	?
	.spc		dd	?
	.bps		dd	?
	.free_clus	dd	?
	.tot_clus	dd	?
	.disk_left	dd	?
	.LowName	rb	16
	.WinFD		WIN32_FIND_DATA
	.sfo		SHFILEOPSTRUCT
endl
;find oldest dir, and try to delete
;then compare new .disk_left with old .disk_left
;if dif., compare with .lowSize
;if not enough, find next oldest dir, and try to delete
.free_dir:
	mov		[.First_time], 0
	lea		eax, [.WinFD]
	invoke	FindFirstFile, FindFileTemp, eax
	cmp		eax, -1
	je		.dir_out						;get out with error set
		mov	[.hSearch], eax
.check_and_compare:
		mov	eax, [.WinFD.dwFileAttributes]
		and	eax, FILE_ATTRIBUTE_DIRECTORY	;is it a directory
		jz	.find_anot						;not a directory
			cld
			lea esi, [.WinFD.cFileName]		;
			lea edi, [.LowName]				;get ready for first time or not
			mov ecx, 12						;
			cmp	[.First_time], 0
			jne	.not_first
				rep movsb					;mov the first time
				mov	[.First_time], 1
				jmp	.find_anot
.not_first:
	repe	cmpsb					;compare when nor first time esi - edi
	jae		.find_anot				;if .LowName(edi) is already < new name, don't copy
		lea esi, [.WinFD.cFileName]		;
		lea edi, [.LowName]				;get ready for first time or not
		mov ecx, 12						;
		rep movsb					;mov the first time
	
.find_anot:
	lea		eax, [.WinFD]
	invoke	FindNextFile, [.hSearch], eax
	cmp		eax, 0
	je		.disp_lowest
		jmp		.check_and_compare
.disp_lowest:
	invoke	lstrcpy, str_buf, [cl_args]
	lea		eax, [.LowName]
	invoke	lstrcat, str_buf, eax
	invoke	lstrcat, str_buf, delete_wildcard
	mov		eax, [hwndEdit]
	mov		[.sfo.hwnd], eax
	mov		[.sfo.wFunc], FO_DELETE
	mov		[.sfo.pFrom], str_buf
	mov		[.sfo.pTo], NULL
	mov		[.sfo.fFlags], FOF_NOERRORUI or FOF_SIMPLEPROGRESS or FOF_ALLOWUNDO or FOF_NOCONFIRMATION
	lea		eax, [.sfoabort]
	mov		[.sfo.fAnyOperationsAborted], eax
	lea		eax, [szRemProject]
	mov		[.sfo.lpszProgressTitle], eax
	mov		[.sfo.hNameMappings], NULL
	lea		eax, [.sfo]
	invoke	SHFileOperation, eax			;Returns zero if sucessful
	cmp		eax, 0
	jne		.couldnt_del

		invoke	lstrcpy, str_buf, [cl_args]
		lea		eax, [.LowName]
		invoke	lstrcat, str_buf, eax
		invoke	RemoveDirectory, str_buf		;Returns zerp if unsucessful
		cmp		eax, 0
		je		.couldnt_del

			invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, dir_deld
			invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, str_buf
			invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, crlf
;		invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, crlf

			mov		edi, cl_args
			lea		eax, [.spc]
			lea		ecx, [.bps]
			lea		edx, [.free_clus]
			lea		esi, [.tot_clus]
	        invoke  GetDiskFreeSpace, dword [edi], eax, ecx, edx, esi
            mov eax, [.bps]
            mul [.free_clus]
            mul [.spc]
            mov [.disk_left], eax
			mov		eax, [.disk_left]
			cmp		eax, [lowSize]
			ja		.enough_room
				jmp		.free_dir

.couldnt_del:
;		invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, str_buf
;		invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, crlf
	mov		eax, 1
	jmp		.dir_out

;Files would be a wild card path such as C:TargetDir*.*

;RootPath is the folder to remove ie: C:TargetDir
;		invoke	MessageBox, [hwndEdit], str_buf, NULL, MB_OK
.enough_room:
	mov		eax, 0					;sucess
.dir_out:
	ret
FreeSome	endp

thread_procedure	proc
		; +------------------+
		; | thread procedure |
		; +------------------+
		mov	ecx, 0x3FFFFFFF
		loopSTART:
			add	eax,eax
			dec	ecx
			jz	loopEND
			jmp	loopSTART
		loopEND:
			invoke	SendMessage, [hWindow], WMU_THREAD_FINISH, NULL, NULL
		ret
thread_procedure	endp

get_cl	proc
local	len_cl	dd	?
;Requires the following routines
; skip_sp
; pro_q
;Requires the following data
;cl_args		dword   80 dup  (?)     ;array of addresses of an up to 80 arguments
;end_of_cl		dword   ?               ;address of end of command line
;lpCmdLine		LPSTR   ?               ;address of command line string
;no! len_cl		dword   ?               ;length of command line
;num_args		dword   ?               ;numbers of cl arguments not counting program

		push	ebx edi esi
		invoke	GetCommandLine		; get the address of the command line
			mov	[lpCmdLine], eax	;store the address
		xor 	al, al				;zero al
		mov 	[num_args], 0
		xor 	ebx, ebx			;pointer to array in cl_args
		mov 	ecx, 10000			;ecx will hold count down from 10000
		mov 	edi, [lpCmdLine]	;destination of scan in EDI
		cld							;advance after each comparison
		repne	scasb				;compare command line bytes to 0
		mov		[end_of_cl], edi	;store add. of end of command line
		dec		[end_of_cl]
		mov		eax, 10000			;original count in ecx
		sub		eax, ecx			;total length of command line including program
		mov		[len_cl], eax		;store total length of command line
		mov		ecx, eax
;If the first character of command line is ' or ", check for the first character which is not ' or "
		mov		edi, [lpCmdLine]
		dec		edi
.nextsqdq:
		inc		edi
		cmp		byte [edi], '"'
		je		.nextsqdq
		cmp		byte [edi], "'"
		je		.nextsqdq
;At this point, edi points to the first non- ' " in command line, if edi == lpCmdLine then there were no ' ", check
		cmp		edi, [lpCmdLine]
		je		.noexquote
;We're here is the first characters of CL were ' or "
		mov		ecx, [end_of_cl]
		sub		ecx, edi
		cmp		byte [edi-1], '"'
		jne		.pro_sq
		mov		al, '"'
		jmp		.find_match
.pro_sq:
		mov		al, "'"
.find_match:
		repne	scasb
		jecxz	.no_args			;if ECX is 0, no args
.noexquote:
		mov		al, ' '				;now check for a ' ' space
;		mov		edi, [lpCmdLine]
		repne	scasb
		jecxz	.no_args			;if ECX is 0, no args
.pro_arg:
		call	skip_sp				;skip all extra spaces, edi returns pointing
									;to first non-space after program
		cmp		byte [edi], 0		;see if this is end of command line
		je		.no_mo_args
		cmp		byte [edi], 34		;is this next character a double quote "
		jne		.c_sq				;if not a dq check for a single quote
		push	34
		call	pro_q				;process a double quote
;		.IF ( eax == FALSE )
;			jmp	pro_ra
;		.ENDIF
		cmp		eax, FALSE
		je		.pro_ra
		jmp		.pro_arg			;process another argument
.c_sq:
		cmp		byte [edi], 39	;is this next character a single quote '
		jne		.pro_ra				;if not a sq process regular argument
		push	39
		call	pro_q				;process a single quote
;		.IF ( eax == FALSE )
;			jmp pro_ra
;		.ENDIF
		cmp		eax, FALSE
		je		.pro_ra
		jmp		.pro_arg				;process another argument
	.pro_ra:							;edi points to something other than
									;0, ' '. '"', "'"
		mov		dword [cl_args + ebx], edi	;beginning of first arg in cl_args
		inc		[num_args]
		add		ebx, 4				;use ebx to point to next pointer in cl_args
		mov		al, ' '				;look for a space in the rest of the cl
		mov		ecx, [end_of_cl]
		sub		ecx, edi			;number of bytes left to process
		repne	scasb				;scan for dq, 34, still in al
		jecxz	.no_mo_args				;jump if a space is not found
		mov		byte [edi - 1], 0
		jmp		.pro_arg
	.no_mo_args:
		cmp		[num_args],   0
		je		.no_args
		jmp		.iep
	.no_args:
	.iep:
		pop		esi edi ebx
		ret
get_cl	endp

clear_quotes	proc
;	Removes quotes around "quoted" arguments changing them to zero teminated strings
;	This occurs if first character of argument is a " or a '
		push	ebx edi
		mov		ecx, [num_args]
		mov		ebx, 0
.anot_cmp:
		mov		edi, [cl_args + ebx]		;points to current arg
		cmp		byte [edi], '"'
		je		.cq
		cmp		byte [edi], "'"
		jne		.anot_arg
.cq:	mov		byte [edi], " "				;overwrite beginning quote
		add		ebx, 4
		mov		edi, [cl_args + ebx]		;points to next arg
		sub		edi, 2						;points to last character of current arg
		mov		byte [edi], " "
		sub		ebx, 4
.anot_arg:
		dec		ecx
		je		.dir_out
		add		ebx, 4
		jmp		.anot_cmp
.dir_out:
		pop		edi ebx
	ret

clear_quotes	endp

skip_sp	proc
									;EDI is pointing to next character in com_line
	c_again:
		cmp	byte [edi], ' '		;compare with space
;		.IF !ZERO?
;			return
;		.ENDIF
		jne	.get_out
		inc	edi
		jmp	c_again
.get_out:
		ret
skip_sp	endp

pro_q	proc type_q
										;edi points to an un-open single quote
										;scan to see if there is a matching sq
										;if not, consider it part of an arg
		push	edi						;store edi in case there is no match
		mov		ecx, [end_of_cl]
		sub		ecx, edi				;number of bytes to process
		inc		edi						;point to next char
;		mov		al, type_q
		mov		eax, [type_q]
		repne	scasb					;scan for sq, 39, still in al
		jecxz	.psq_ra					;jump if sq not found in rest of cl
		mov		byte [edi - 1], 0		;replace last sq with a 0 for end of string
		pop		esi
		mov		byte [esi], 0			;replace first sq with a 0
		inc		esi
		mov		[cl_args + ebx], esi	;beginning of first arg in cl_args
		inc		[num_args]
		add		ebx, 4					;use ebx to point to next pointer in cl_args
;       set edi to end of this arg!!!
		mov		eax, TRUE				;return TRUE
		jmp		.dir_out
.psq_ra:
		pop		edi
		mov		eax, FALSE				;second quote not found, process as a regular argument
.dir_out:
		ret
pro_q	endp

OnlyOneInstance	proc
locals
	hSemaphore	dd	?
	hTemp		dd	?
endl
	invoke	CreateSemaphore, NULL, 0, 1, wTitle
		mov	[hSemaphore], eax
	invoke	GetLastError
		cmp	eax, ERROR_ALREADY_EXISTS
		je	.cancel_create
			;we're here if we want to create a new instance
			mov	eax, TRUE		; create new instance
			jmp	.dir_out		; return TRUE to continue
.cancel_create:
	invoke	CloseHandle, [hSemaphore]	;Close the one we just created
	invoke	FindWindow, wClsName, wTitle
	cmp		eax, NULL
	je		.ReturnZero
	invoke	GetLastActivePopup, eax
	mov		[hTemp], eax					; eax == hMainWnd or hPopupWnd

	invoke	IsIconic, [hTemp]
	cmp		eax, TRUE
	jne		.already_big
		invoke	ShowWindow, [hTemp], SW_RESTORE
		jmp		.ReturnZero				; return FALSE (eax=0) to exit
.already_big:
	invoke	SetForegroundWindow, [hTemp]
	invoke	SetFocus, [hTemp]

.ReturnZero:
	mov		eax, FALSE		; return FALSE (eax=0) to exit
.dir_out:	
	ret
OnlyOneInstance	endp

get_dt	proc
locals
	.temp	rd	2
	.ltd	SYSTEMTIME
endl
;We need--for 20050327SUN--year, month, day, & DOW, put in str_buf

	push	ebx edi
	
	lea		edi, [str_buf]
	lea		eax, [.ltd]
	invoke	GetLocalTime, eax					;
	movzx	eax, [.ltd.wYear]
;	push	0						;don't add commas
;	push	edi
;	push	eax
;	call	jdwtoa
	stdcall	jdwtoa, eax, edi, 0
;	cmp		[.ltd.wMonth], 10
	movzx	eax, [.ltd.wMonth]
	cmp		eax, 10
	jae		.no_zero_mon
		mov		byte [edi + 4], '0'
		jmp		.day
.no_zero_mon:
	mov		byte [edi + 4], '1'
	sub		eax, 10
.day:
	add		eax, '0'
	mov		byte [edi + 5], al
	movzx	eax, [.ltd.wDay]
	mov		ecx, 10
	div		cl
	add		al, '0'
	mov		byte [edi + 6], al
	add		ah, '0'
	mov		byte [edi + 7], ah

	movzx	eax, [.ltd.wDayOfWeek]				;Change to movxz eax
	mov		bx, 03h
	mul		bx
	cld
	mov		esi, days
	add		esi, eax							;SO THAT THIS, WOULD ADD CORRECT eax
	add		edi, 8
	mov		ecx, 3
	rep		movsb								;error at heathers
	
	lea		edi, [str_buf]
	mov		byte [edi + 11], 0
	
	invoke	CreateDirectory, edi, NULL			;returns zero if unsucessful
	
	pop		edi ebx
	ret
get_dt	endp

ProcessFiles	proc
locals
	.cnt_n		dd	?
	.fhandl		dd	?
	.filestru	WIN32_FIND_DATA
endl
	push	ebx edi esi
	mov		[.cnt_n], 1									;Process file spec #1
.ano_arg:
	mov		eax, [num_args]
	sub		eax, [.cnt_n]
	jle		.done_arg
	mov		eax, [.cnt_n]								;Look into cl_args array
	shl		eax, 2										;Multiply by 4
	mov		ebx, [cl_args + eax]						;address of file spec #cnt_n
	mov		[arg_n], ebx								;arg_n is address of argument
	call	set_dir
	inc		[.cnt_n]									;get ready to process next parameter
	invoke	SetCurrentDirectory, pathoffile				;returns 0 if unsucessful
	cmp		eax, 0
	jne		.no_change
;	.IF (eax == FALSE)									;Could not change to requested directory
		invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, crlf
		invoke	SendMessage, [hwndEdit], EM_SETSEL, -1, -1
		invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, cncdirectory
		invoke	SendMessage, [hwndEdit], EM_SETSEL, -1, -1
		invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, pathoffile
		invoke	SendMessage, [hwndEdit], EM_SETSEL, -1, -1
		invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, crlf
		jmp		.ano_arg
;	.ENDIF
.no_change:
	cld
	mov		edi, finalname
	mov		esi, [arg_n]
	add		esi, [pathsize]
	inc		esi
	mov		eax, [src_size]
	sub		eax, [pathsize]
	dec		eax
	mov		ecx, eax
	rep		movsb
	mov		[edi], byte 0
	lea		eax, [.filestru]
	invoke	FindFirstFile, finalname, eax		;returns -1 if unsucessful
	cmp		eax, INVALID_HANDLE_VALUE
	jne		.file_found
;	.IF (eax == INVALID_HANDLE_VALUE)
		invoke	SendMessage, [hwndEdit], EM_SETSEL, -1, -1
		invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, cnffile
		invoke	SendMessage, [hwndEdit], EM_SETSEL, -1, -1
		invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, finalname
		invoke	SendMessage, [hwndEdit], EM_SETSEL, -1, -1
		invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, crlf
		jmp		.ano_arg
;	.ENDIF
.file_found:
	mov		[.fhandl], eax
	mov		edi, destfile
	mov		esi, drive
	mov		ecx, 14
	cld
	rep		movsb
	mov		[edi], byte '\'
;	.REPEAT
.aff:
		mov		ecx, MAX_PATH
		sub		ecx, 16
		lea		esi, [.filestru.cFileName]
		mov		edi, destfile
		add		edi, 15
		rep		movsb
		lea		eax, [.filestru.cFileName]
		invoke	CopyFile, eax, destfile, FALSE
			cmp eax, TRUE
			jne	.nf
;		.IF (eax == TRUE)
			invoke	SendMessage, [hwndEdit], EM_SETSEL, -1, -1
			invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, fbackedup
			invoke	SendMessage, [hwndEdit], EM_SETSEL, -1, -1
			lea		eax, [.filestru.cFileName]
			invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, eax
			invoke	SendMessage, [hwndEdit], EM_SETSEL, -1, -1
			invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, crlf
			jmp		.faf
;		.ELSE
.nf:
			invoke	SendMessage, [hwndEdit], EM_SETSEL, -1, -1
			invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, fnbackedup
			invoke	SendMessage, [hwndEdit], EM_SETSEL, -1, -1
			lea		eax, [.filestru.cFileName]
			invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, eax
			invoke	SendMessage, [hwndEdit], EM_SETSEL, -1, -1
			invoke	SendMessage, [hwndEdit], EM_REPLACESEL, TRUE, crlf
;		.ENDIF
.faf:
		lea		eax, [.filestru]
		mov		ebx, [.fhandl]
		invoke	FindNextFile, ebx, eax				;returns 0 if not sucessful
		cmp	eax, 0
	jne	.aff
;	.UNTIL (!eax)
	jmp		.ano_arg
.done_arg:
	mov		ebx, [.fhandl]
	invoke	FindClose, ebx

	mov		eax, 0

	pop	esi edi ebx
	ret
ProcessFiles	endp

set_dir	proc	

	
	push	ebx edi esi
;DIRECTORY STRING IS IN retstring
;sourcefile CONTAINS THE DRIVE + FILENAME STRING
	mov		ebx, 0
	mov		edi, [arg_n]
.find0:
	cmp		byte [edi + ebx], 0			;look for end of string
	je		.found0						;if found jmp
	inc		ebx							;keep count in ebx
	jmp		.find0						;look at next byte
.found0:
	mov		[src_size], ebx				;keep track of size of source string
	dec		ebx							;point to last character in source
.findlbs:
	cmp		byte [edi + ebx], '\'		;look for back slash
	je		.foundlbs					;if found jmp
	dec		ebx							;look a previous byte
	jg		.findlbs					;if not zero, look
.foundlbs:
	mov		ecx, ebx					;ebx contains the position of last back
										;slash in sourcefile string
	mov		[pathsize], ebx
	cld
	mov		edi, pathoffile
	mov		esi, [arg_n]
	rep		movsb
	mov		[edi], byte 0				;directory of source files at pathoffile

	pop		esi edi ebx

	ret
set_dir	endp

section '.data'		data		readable	writeable
	IncludeAllGlobals 

section '.idata'	import data readable writeable
	library		KERNEL32,	'KERNEL32.DLL',\
				USER32,		'USER32.DLL',\
				SHELL32,	'SHELL32.DLL'

	import		KERNEL32,\
					GetModuleHandle,	'GetModuleHandleA',\
					ExitProcess,		'ExitProcess',\
					CreateThread,		'CreateThread',\
					CloseHandle,		'CloseHandle',\
					GetLastError,		'GetLastError',\
					FormatMessage,		'FormatMessageA',\
					LocalFree,			'LocalFree',\
					GetCommandLine,		'GetCommandLineA',\
					CreateSemaphore,	'CreateSemaphoreA',\
					SetCurrentDirectory,'SetCurrentDirectoryA',\
					CreateFile,			'CreateFileA',\
					CopyFile,			'CopyFileA',\
					GetFileSize,		'GetFileSize',\
					GetDiskFreeSpace,	'GetDiskFreeSpaceA',\
					GetDiskFreeSpaceEx,	'GetDiskFreeSpaceExA',\
					FindFirstFile,		'FindFirstFileA',\
					FindNextFile,		'FindNextFileA',\
					FindClose,			'FindClose',\
					lstrcpy,			'lstrcpy',\
					lstrcat,			'lstrcat',\
					RemoveDirectory,	'RemoveDirectoryA',\
					GetLocalTime,		'GetLocalTime',\
					CreateDirectory,	'CreateDirectoryA',\
					SetErrorMode,		'SetErrorMode'

	import		SHELL32,\
					SHFileOperation,	'SHFileOperation'

	import		USER32,\
					RegisterClass,		'RegisterClassA',\
					CreateWindowEx,		'CreateWindowExA',\
					DefWindowProc,		'DefWindowProcA',\
					LoadCursor,			'LoadCursorA',\
					LoadIcon,			'LoadIconA',\
					GetMenu,			'GetMenu',\
					MessageBox,			'MessageBoxA',\
					SendMessage,		'SendMessageA',\
					GetMessage,			'GetMessageA',\
					DestroyWindow,		'DestroyWindow',\
					TranslateMessage,	'TranslateMessage',\
					DispatchMessage,	'DispatchMessageA',\
					PostQuitMessage,	'PostQuitMessage',\
					GetSystemMetrics,	'GetSystemMetrics',\
					ShowWindow,			'ShowWindow',\
					UpdateWindow,		'UpdateWindow',\
					SetFocus,			'SetFocus',\
					FindWindow,			'FindWindowA',\
					GetLastActivePopup,	'GetLastActivePopup',\
					IsIconic,			'IsIconic',\
					SetForegroundWindow,	'SetForegroundWindow',\
					SetTimer,			'SetTimer',\
					KillTimer,			'KillTimer'

section '.rsrc'		resource data readable

EditID					=	2
ERROR_ALREADY_EXISTS	=	183
FO_DELETE				=	3
FOF_MULTIDESTFILES		=	1
FOF_CONFIRMMOUSE		=	2h
FOF_SILENT				=	4h
FOF_RENAMEONCOLLISION	=	8h
FOF_NOCONFIRMATION		=	10h
FOF_WANTMAPPINGHANDLE	=	20h
FOF_ALLOWUNDO			=	40h
FOF_FILESONLY			=	80h
FOF_SIMPLEPROGRESS		=	100h
FOF_NOCONFIRMMKDIR		=	200h
FOF_NOERRORUI			=	400h 
FOF_NOCOPYSECURITYATTRIBS	=	800h 
	directory	RT_MENU, appMenu

	resource	appMenu,\
					30, LANG_NEUTRAL, menuMain

	menu	menuMain
			menuitem	'E&xit', MI_EXIT, MFR_END

;		jmp		.cont_p
;		jmp		.cont_p
;		push	edi
;		push	ebx
;		push 	[num_args]
;		mov		ebx, 0
;		mov		edi, cl_args
;.disp_args:
;		invoke	MessageBox, 0, dword [edi + ebx], NULL, MB_OK
;		add		ebx, 4
;		dec		[num_args]
;		jne		.disp_args
;		pop		[num_args]
;		pop		ebx
;		pop		edi
;		jmp		.cont_p
;
